# -*- coding: utf-8 -*-
"""
@date Created on Fri May 22 2020
@author martin_g for Eomys
"""

# Standard library imports
import numpy as np
from scipy import signal


def _square_and_smooth(sig, center_freq, fs):
    """3rd order low-pass filtering (See ISO 532-1 section 6.3)

    Parameters
    ----------
    sig : numpy.ndarray
        time signal sampled at 48 kHz [pa]
    coeff : numpy.ndarray
        filter coeeficients
    gain : float
        filter gain

    Outputs
    -------
    signal_filt : numpy.ndarray
        filtered time signal
    """
    # Frequency dependent time constant
    if center_freq <= 1000:
        tau = 2 / (3 * center_freq)
    else:
        tau = 2 / (3 * 1000)
    # Squaring
    sig = sig ** 2
    # Three smoothing low-pass filters
    a1 = np.exp(-1 / (fs * tau))
    b0 = 1 - a1
    # zi = signal.lfilter_zi([b0], [1 -a1])
    for i in range(3):
        sig = signal.lfilter([b0], [1, -a1], sig)
    return sig



# -*- coding: utf-8 -*-

# Standard library imports
import numpy as np

# local import
from mosqito.sound_level_meter.noct_spectrum._center_freq import _center_freq


def _filter_bandwidth(fc, n=3, N=3):
    """Define nth octave filter bandwidth

    The filter bandwidth is designed according to the Order-N
    specification of the ANSI S1.1-1986 standard. Default
    value for N is 3.

    References:
        ANSI S1.1-1986 (ASA 65-1986): Specifications for
        Octave-Band and Fractional-Octave-Band Analog and
        Digital Filters.

    Parameters
    ----------
    fc : ndarray
        Filters exact center frequencies
    n : int, optional
        Number of bands pr octave. Default to 3
    N : int, optional
        Filter order. Default to 3

    Outputs
    -------
    alpha : numpy.ndarray
        Ratio of the upper and lower band-edge frequencies to the mid-band
        frequency
    """

    """
    TODO: Check compliancy with the following standards
        - IEC 61260 – 1 (2014), 1/1-octave Bands and 1/3-octave Bands, Class 1
        - IEC 61260 (1995 – 07) plus Amendment 1 (2001 – 09), 1/1-octave Bands and 1/3-octave Bands, Class 0
        - ANSI S1.11 – 1986, 1/1-octave Bands and 1/3-octave Bands, Order 3, Type 0 – C
        - ANSI S1.11 – 2004, 1/1-octave Bands and 1/3-octave Bands, Class 0
        - ANSI/ASA S1.11 – 2014 Part 1, 1/1-octave Bands and 1/3-octave Bands, Class 1
    """

    # Design for Butterworth 2Nth-order nth-octave filter
    # Note: BUTTER is based on a bilinear transformation, as suggested in
    # ANSI S1.1-1986.
    b = 1 / n
    f1 = fc / (2 ** (b / 2))  # ANSI eq5
    f2 = fc * (2 ** (b / 2))  # ANSI eq6
    # Reference bandwidth quotient
    qr = fc / (f2 - f1)  # ANSI eq7 & 8
    # Design bandwidth quotient
    qd = (np.pi / 2 / N) / (np.sin(np.pi / 2 / N)) * qr  # ANSI eq9
    # Ratio of the upper and lower band-edge frequencies to the mid-band frequency
    alpha = (1 + np.sqrt(1 + 4 * qd ** 2)) / 2 / qd

    return alpha


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-

import numpy as np


def time_segmentation(sig, fs, nperseg=2048, noverlap=None, is_ecma=False):
    """Function used for the segmentation of a time signal into
    smaller parts of audio (blocks).

    Parameters
    ----------
    sig: numpy.array
        A 1-dimensional time signal array.
    fs : float
        The time signal sampling frequency.
    nperseg: int, optional
        Length of each segment. Defaults to 2048.
    noverlap: int, optional
        Number of points to overlap between segments.
        If None, noverlap = nperseg / 2. Defaults to None.
    is_ecma: bool, optional
        If is_ecma = True, the implementation is as described
        in Formula 16 (section 5.1.4) of ECMA-418-2. Defaults
        to False.

    Returns
    -------
    block_array: numpy.array
        A 2-dimensional array of size (nperseg, nseg)
        containing the segmented signal.
    time: numpy.array
        The time axis corresponding to the segmented
        signal, size (nseg,)
    """

    if noverlap is None:
        noverlap = int(nperseg / 2)

    if is_ecma:
        # pad with zeros at the begining
        sig = np.hstack((np.zeros(nperseg), sig))

    # build time axis for sig
    time = np.linspace(0, (len(sig) - 1) / fs, num=len(sig))

    l = 0
    block_array = []
    time_array = []
    while l * noverlap <= len(sig) - nperseg:
        block_array.append(sig[l * noverlap : nperseg + l * noverlap])
        time_array.append(np.mean(time[l * noverlap : nperseg + l * noverlap]))
        l += 1

    return np.array(block_array).T, np.array(time_array)



# -*- coding: utf-8 -*-
"""
Created on Mon Nov  2 10:34:04 2020

@author: wantysal
"""

# Standard imports
import numpy as np
from numpy.fft import fft, ifft
import math

# Local imports
from mosqito.utils.LTQ import LTQ
from mosqito.sq_metrics.roughness.roughness_dw._gzi_weighting import (
    _gzi_weighting,
)
from mosqito.sq_metrics.roughness.roughness_dw._H_weighting import _H_weighting
from mosqito.sq_metrics.roughness.roughness_dw._ear_filter_coeff import (
    _ear_filter_coeff,
)
from mosqito.utils.conversion import freq2bark, db2amp, amp2db, bark2freq


def roughness_dw(signal, fs, overlap):
    """Roughness calculation of a signal sampled at 48kHz.

    The code is based on the algorithm described in "Psychoacoustical roughness:
    implementation of an optimized model" by Daniel and Weber in 1997.
    The roughness model consists of a parallel processing structure that is made up
    of successive stages and calculates intermediate specific roughnesses R_spec,
    which are summed up to determine the total roughness R.

    Parameters
    ----------
    signal : numpy.array
        signal amplitude values along time
    fs : integer
        sampling frequency
    overlap : float
        overlapping coefficient for the time windows of 200ms

    Outputs
    -------
    R : numpy.array
        roughness
    time : numpy.array
           time axis

    """

    # -----------------------------------Stage 0------------------------------------
    # -------Creation of overlapping frames of 200 ms from the input signal---------

    # Number of points within each frame according to the time resolution of 200ms
    n = int(0.2 * fs)

    # Number of frames according to the given overlap proportion
    nb_frame = math.floor(signal.size / ((1 - overlap) * n)) - 1

    if nb_frame == 0:
        nb_frame = 1

    # Creation of the corresponding time axis
    time = np.linspace(0, len(signal) / fs, num=nb_frame)

    # Initialization of the weighting functions H and g
    hWeight = _H_weighting(n, fs)
    # Aures modulation depth weighting function
    gzi = _gzi_weighting(np.arange(1, 48, 1) / 2)

    R = np.zeros((nb_frame))
    for i_frame in range(nb_frame):
        segment = signal[
            i_frame * int(n * (1 - overlap)): i_frame * int(n * (1 - overlap)) + n
        ]

        # Calculate Blackman analysis window
        window = np.blackman(n)
        window = window / sum(window)

        # Creation of the spectrum by FFT using the Blackman window
        spectrum = fft(segment * window) * 1.42
        # Highest frequency
        nMax = round(n / 2)
        nZ = np.arange(1, nMax + 1, 1)
        # Frequency axis in Hertz
        freqs = np.arange(1, nMax + 1, 1) * (fs / n)
        # Frequency axis in Bark
        barks = freq2bark(freqs)

        # Calculate Zwicker a0 factor (transfer characteristic of the outer and inner ear)
        a0 = np.zeros((n))
        a0[nZ - 1] = db2amp(_ear_filter_coeff(barks), ref=1)
        spectrum = a0 * spectrum

        # Conversion of the spectrum into dB
        module = abs(spectrum[0:nMax])
        spec_dB = amp2db(module, ref=0.00002)

        # Find the audible components within the spectrum
        threshold = LTQ(barks, reference="roughness")
        audible_index = np.where(spec_dB > threshold)[0]
        # Number of audible frequencies
        n_aud = len(audible_index)

        # --------------------------------- stage 1 ------------------------------------
        # ----------------Creation of the specific excitations functions----------------

        # Terhardt's slopes definition
        # lower slope [dB/Bark]
        s1 = -27
        s2 = np.zeros((n_aud))
        # upper slope [dB/Bark]
        for k in np.arange(0, n_aud, 1):
            s2[k] = min(
                -24
                - (230 / freqs[audible_index[k]])
                + (0.2 * spec_dB[audible_index[k]]),
                0,
            )

        # The excitation pattern are calculated for 47 overlapping 1-bark-wide channels
        n_channel = 47
        # Channels number
        zi = np.arange(1, n_channel + 1) / 2
        # Center frequencies for each channel
        zb = bark2freq(zi) * n / fs
        # Minimum excitation level
        minExcitDB = np.interp(zb, nZ, threshold)

        ch_low = np.zeros((n_aud))
        ch_high = np.zeros((n_aud))
        for i in np.arange(0, n_aud):
            # Lower limit of the channel corresponding to each component
            ch_low[i] = math.floor(2 * barks[audible_index[i]]) - 1
            # Higher limit
            ch_high[i] = math.ceil(2 * barks[audible_index[i]]) - 1

        # Creation of the excitation pattern
        slopes = np.zeros((n_aud, n_channel))
        for k in np.arange(0, n_aud):
            levDB = spec_dB[audible_index[k]]
            b = barks[audible_index[k]]
            for j in np.arange(0, int(ch_low[k] + 1)):
                sl = (s1 * (b - ((j + 1) * 0.5))) + levDB
                if sl > minExcitDB[j]:
                    slopes[k, j] = db2amp(sl, ref=0.00002)
            for j in np.arange(int(ch_high[k]), n_channel):
                sl = (s2[k] * (((j + 1) * 0.5) - b)) + levDB
                if sl > minExcitDB[j]:
                    slopes[k, j] = db2amp(sl, ref=0.00002)

        # Initialization
        hBP = np.zeros((n_channel, n))
        mod_depth = np.zeros((n_channel))

        # Definition of the excitation amplitude
        for i in np.arange(0, n_channel, 1):
            exc = np.zeros((n), dtype=complex)
            for j in np.arange(0, n_aud, 1):
                ind = audible_index[j]
                # the component belongs to the bark window
                if ch_low[j] == i:
                    ampl = 1
                elif ch_high[j] == i:
                    ampl = 1
                # the component is higher than the bark window
                elif ch_high[j] > i:
                    ampl = slopes[j, i + 1] / module[ind]
                # the component is lower than the considered window
                else:
                    ampl = slopes[j, i - 1] / module[ind]

                # reconstruction of the spectrum
                exc[ind] = ampl * spectrum[ind]

            # The temporal specific excitation functions are obtained by IFFT
            temporal_excitation = np.abs(n * np.real(ifft(exc)))

            # ------------------------------- stage 2 --------------------------------------
            # ---------------------modulation depth calculation-----------------------------

            # The fluctuations of the envelope are contained in the low frequency part
            # of the spectrum of specific excitations in absolute value
            h0 = np.mean(temporal_excitation)
            envelope_spec = fft(temporal_excitation - h0)

            # This spectrum is weighted to model the low-frequency  bandpass
            # characteristic of the roughness on modulation frequency
            envelope_spec = envelope_spec * hWeight[i, :]

            # The time functions of the bandpass filtered envelopes hBPi(t)
            # are calculated via inverse Fourier transform :
            hBP[i, :] = 2 * np.real(ifft(envelope_spec))

            # Modulation depth estimation is given by envelope RMS values
            # and excitation functions time average :
            hBPrms = np.sqrt(np.mean(np.power(hBP[i, :], 2)))
            if h0 > 0:
                mod_depth[i] = hBPrms / h0
                if mod_depth[i] > 1:
                    mod_depth[i] = 1
            else:
                mod_depth[i] = 0

        # ------------------------------- stage 3 --------------------------------------
        # ----------------roughness calculation with cross correlation------------------

        # Crosscorrelation coefficients between the envelopes of the channels
        # i and i+2 with dz= 1 bark
        ki = np.zeros((47))

        for i in range(0, 45):
            if hBP[i].all() != 0 and hBP[i + 2].all() != 0:
                ki[i] = np.corrcoef(hBP[i, :], hBP[i + 2, :])[0, 1]

        # Specific roughness calculation with gzi the modulation depth weighting
        # function given by Aures
        R_spec = np.zeros((47))

        R_spec[0] = gzi[0] * pow(mod_depth[0] * ki[0], 2)
        R_spec[1] = gzi[1] * pow(mod_depth[1] * ki[1], 2)
        for i in np.arange(2, 45):
            R_spec[i] = gzi[i] * pow(mod_depth[i] * ki[i] * ki[i - 2], 2)
        R_spec[45] = gzi[45] * pow(mod_depth[45] * ki[43], 2)
        R_spec[46] = gzi[46] * pow(mod_depth[46] * ki[44], 2)

        # Total roughness calculation with calibration factor of 0.25 given in the article
        # to produce a roughness of 1 asper for a 1-kHz, 60dB tone with carrier frequency
        # of 70 Hz and a modulation depth of 1

        R[i_frame] = 0.25 * sum(R_spec)

    return R, time



# -*- coding: utf-8 -*-

# Optional package import
try:
    import pytest
except ImportError:
    raise RuntimeError(
        "In order to perform the tests you need the 'pytest' package."
        )
import numpy as np

# Local application imports
from mosqito.utils.sine_wave_generator import (
    sine_wave_generator,
)
from mosqito.sq_metrics import loudness_ecma


@pytest.mark.loudness_ecma  # to skip or run only loudness ecma tests
def test_loudness_ecma():
    """Test function for the Loudness_ecma calculation"""

    # Generate a 1kHz / 80 dB test tone and compute loudness
    signal, _ = sine_wave_generator(
        fs=48000,
        t=0.25,
        spl_value=80,
        freq=1000,
    )
    n_specific, _ = loudness_ecma(signal)
    n_specific = np.array(n_specific)
    n_tot = np.sum(n_specific, axis=0)
    n_tot_mean_1kHz = np.mean(n_tot[5:])

    # Generate a 5kHz / 78.7 dB test tone and compute loudness
    signal, _ = sine_wave_generator(
        fs=48000,
        t=0.25,
        spl_value=78.73977248964925,
        freq=5000,
    )
    n_specific, _ = loudness_ecma(signal)
    n_specific = np.array(n_specific)
    n_tot = np.sum(n_specific, axis=0)
    n_tot_mean_5kHz = np.mean(n_tot[5:])

    # Compare loudness
    np.testing.assert_almost_equal(n_tot_mean_1kHz, n_tot_mean_5kHz)


# test de la fonction
if __name__ == "__main__":
    test_loudness_ecma()



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 16 20:23:01 2020

@author: wantysal
"""
# Standard library imports
import numpy as np

# Mosqito functions import
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._spectrum_smoothing import (
    _spectrum_smoothing,
)
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._LTH import _LTH
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._critical_band import _critical_band


def _screening_for_tones(freqs, spec_db, method, low_freq, high_freq):
    """
        Screening function to find the tonal candidates in a spectrum

        The 'smoothed' method is the one described by Bray W and Caspary G in :
        Automating prominent tone evaluations and accounting for time-varying
        conditions, Sound Quality Symposium, SQS 2008, Detroit, 2008.

        The 'not-smoothed' method is the one used by Aures and Terhardt

        The criteria of tonal width comes from Wade Bray in 'Methods for automating
        prominent tone evaluation and for considering variations with time or other
        reference quantities'

    Parameters
    ----------
    freqs : numpy.array
        frequency axis
    spec_db : numpy.array
        spectrum in dB
    method : string
        the method chosen to find the tones 'Sottek'
    low_freq : float
        lowest frequency of interest
    high_freq : float
        highest frequency of interest


    Returns
    -------
    tones : list
        list of index corresponding to the potential tonal components

    """

    ###############################################################################
    # Detection of the tonal candidates according to their level

    # Creation of the smoothed spectrum
    smooth_spec = _spectrum_smoothing(freqs, spec_db, 24, low_freq, high_freq, freqs)

    if method == "smoothed":

        # Criteria 1 : the level of the spectral line is higher than the level of
        # the two neighboring lines
        maxima = (np.diff(np.sign(np.diff(spec_db))) < 0).nonzero()[0] + 1

        # Criteria 2 : the level of the spectral line exceeds the corresponding lines
        # of the 1/24 octave smoothed spectrum by at least 6 dB
        indexx = np.where(spec_db[maxima] > smooth_spec[maxima] + 6)[0]

        # Criteria 3 : the level of the spectral line exceeds the threshold of hearing
        threshold = _LTH(freqs)
        audible = np.where(spec_db[maxima][indexx] > threshold[maxima][indexx] + 10)[0]

        index = np.arange(0, len(spec_db))[maxima][indexx][audible]

    if method == "not-smoothed":
        # Criteria 1 : the level of the spectral line is higher than the level of
        # the two neighboring lines
        maxima = (
            np.diff(np.sign(np.diff(spec_db[3 : len(spec_db) - 3]))) < 0
        ).nonzero()[
            0
        ] + 1  # local max

        # Criteria 2 : the level of the spectral line is at least 7 dB higher than its
        # +/- 2,3 neighbors
        indexx = np.where(
            (spec_db[maxima] > (spec_db[maxima + 2] + 7))
            & (spec_db[maxima] > (spec_db[maxima - 2] + 7))
            & (spec_db[maxima] > (spec_db[maxima + 3] + 7))
            & (spec_db[maxima] > (spec_db[maxima - 3] + 7))
        )[0]

        # Criteria 3 : the level of the spectral line exceeds the threshold of hearing
        threshold = _LTH(freqs)
        audible = np.where(spec_db[maxima][indexx] > threshold[maxima][indexx] + 10)[0]

        index = np.arange(0, len(spec_db))[maxima][indexx][audible]

    ###############################################################################
    # Check of the tones width : a candidate is discarded if its width is greater
    # than half the critical bandwidth

    tones = np.empty((0), dtype=int)
    # Each candidate is evaluated
    while len(index) > 0:
        # Index of the candidate
        peak_index = index[0]

        # Lower and higher limits of the tone width
        low_limit = peak_index
        high_limit = peak_index

        # Screen the right points of the peak
        temp = peak_index + 1

        # As long as the level decreases or remains above the smoothed spectrum,
        while (spec_db[temp] > smooth_spec[temp] + 6) and (temp + 1 < len(spec_db)):
            # if a highest spectral line is found, it becomes the candidate
            if spec_db[temp] > spec_db[peak_index]:
                peak_index = temp
            high_limit += 1
            temp += 1

        # Screen the left points of the peak
        temp = peak_index - 1
        # As long as the level decreases,
        while (spec_db[temp] > smooth_spec[temp] + 6) and (temp + 1 < len(spec_db)):
            # if a highest spectral line is found, it becomes the candidate
            if spec_db[temp] > spec_db[peak_index]:
                peak_index = temp
            low_limit -= 1
            temp -= 1

        # Critical bandwidth
        f1, f2 = _critical_band(freqs[peak_index])
        cb_width = f2 - f1

        # Tonal width
        t_width = freqs[high_limit] - freqs[low_limit]

        if t_width < cb_width:
            tones = np.append(tones, peak_index)

        # All the candidates already screened are deleted from the list
        sup = np.where(index <= high_limit)[0]
        index = np.delete(index, sup)

    return tones



import numpy as np


def _get_rns_index(array_nm, vector_rns, equal_too=False):
    """Function that returns the index in the array vector_rns for each value of srray _nm

    Parameters
    ----------
    array_nm : numpy.ndarray, values of the matrix toget the indexes

    vector_rns:reference vector to get indexes
    equal_too : boolean

    Outputs
    -------
    indexes :  numpy.ndarray
        Array of indexes
    """

    if len(array_nm.shape) == 1:
        (wide,) = array_nm.shape
        (deep,) = vector_rns.shape
        array_aux = np.round(np.tile(array_nm, [deep, 1]), 8)
        rns_array = np.round(np.ones([wide, deep]) * vector_rns, 8).T
    else:
        wide, length = array_nm.shape
        (deep,) = vector_rns.shape
        array_aux = np.round(np.tile(array_nm, [deep, 1, 1]), 8)
        rns_array = np.round((np.ones([wide, length, deep]) * vector_rns), 8).transpose(
            2, 0, 1
        )
        # indexes = (array_aux < rns_array).sum(axis=0)
        # indexes[indexes==18] = 17
    if equal_too:
        indexes = (array_aux <= rns_array).sum(axis=0)
    else:
        indexes = (array_aux < rns_array).sum(axis=0)
    indexes[indexes == 18] = 17

    return indexes



# -*- coding: utf-8 -*-
"""
@date Created on Fri May 22 2020
@author martin_g for Eomys
"""

# Standard library imports
import math
import numpy as np
#Needed for the loudness_zwicker_lowpass_intp_ea function
from scipy import signal


def _lowpass_intp(loudness, tau, sample_rate):
    """1st order low-pass with linear interpolation of signal for
    increased precision

    Parameters
    ----------
    loudness : numpy.ndarray
        Loudness vs. time
    tau : float
        Filter parameter
    sample_rate : int
        Louness signal sampling frequency

    Outputs
    -------
    filt_loudness : numpy.ndarray
        Filtered loudness
    """
    filt_loudness = np.zeros(np.shape(loudness))
    # Factor for virtual upsampling/inner iterations
    lp_iter = 24

    num_samples = np.shape(loudness)[0]
    a1 = math.exp(-1 / (sample_rate * lp_iter * tau))
    b0 = 1 - a1
    y1 = 0

    delta = np.copy(loudness)
    delta = np.roll(delta,-1)
    delta [-1] = 0
    delta = (delta - loudness) /  lp_iter
    ui_delta = np.zeros(loudness.shape[0]*lp_iter).reshape(loudness.shape[0],lp_iter)
    ui_delta [:,0] = loudness  
    
    #Create the array complete of deltas to apply the filter.
    for i_in in np.arange(1, lp_iter):
        ui_delta [:,i_in] = delta + ui_delta [:,i_in-1]  
    
    # Rechape into a vector.
    ui_delta = ui_delta.reshape(lp_iter*num_samples)

    # Apply the filter.
    ui_delta = signal.lfilter([b0], [1,-a1], ui_delta, axis=- 1, zi=None)
    
    # Reshape again to recover the first col.
    ui_delta = ui_delta.reshape(loudness.shape[0],lp_iter)
    filt_loudness = ui_delta[:,0]
    return filt_loudness



# -*- coding: utf-8 -*-

# Standard library import
import numpy as np

# Local imports
from mosqito.sq_metrics import loudness_zwst
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_from_loudness import (
    sharpness_din_from_loudness,
)


def sharpness_din(
    signal,
    fs,
    method="zwst",
    weighting="din",
    field_type="free",
    skip=0,
):
    """Acoustic sharpness calculation according to different methods:
        Aures, Von Bismarck, DIN 45692, Fastl

    Parameters:
    ----------
    signal: numpy.array
        time history values
    fs: integer
        sampling frequency
    method : string
        To specify the Loudness computation method
    weighting : string
        To specify the weighting function used for the
        sharpness computation.'din' by default,'aures', 'bismarck','fastl'
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse").
    skip : float
        number of second to be cut at the beginning of the analysis

    Outputs
    ------
    S : float
        sharpness value
    time_axis: numpy.array
        The time axis array, size (Ntime,) or None

    """

    # Compute loudness
    if method == "zwst":
        N, N_specific, _ = loudness_zwst(signal, fs, field_type=field_type)
    elif method == "zwtv":
        # TBD
        pass
    else:
        raise ValueError("ERROR: method must be either 'zwst' or 'zwtv'")

    # Compute sharpness from loudness
    S = sharpness_din_from_loudness(N, N_specific, weighting=weighting, skip=0)

    return S



# -*- coding: utf-8 -*-

# Third party imports
import pytest

# Local application imports
from mosqito.sq_metrics import loudness_zwtv
from mosqito.utils import load
from validations.sq_metrics.loudness_zwtv.validation_loudness_zwtv import (
    _check_compliance,
)


@pytest.mark.loudness_zwtv  # to skip or run only loudness zwicker time-varying tests
def test_loudness_zwtv():
    """Test function for the script loudness_zwicker_time

    Test function for the script loudness_zwtv with
    .wav file as input. The input file is provided by ISO 532-1 annex
    B4 and B5, the compliance is assessed according to section 6.1 of the
    standard. One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for time-varying loudness
    # (from ISO 532-1 annex B4)
    signal = {
        "data_file": "tests/input/Test signal 10 (tone pulse 1 kHz 10 ms 70 dB).wav",
        "xls": "tests/input/Results and tests for synthetic signals (time varying loudness).xlsx",
        "tab": "Test signal 10",
        "N_specif_bark": 8.5,
        "field": "free",
    }

    # Load signal and compute third octave band spectrum
    sig, fs = load(signal["data_file"], wav_calib=2 * 2 ** 0.5)

    # Compute Loudness
    N, N_spec, bark_axis, time_axis = loudness_zwtv(sig, fs, signal["field"])
    loudness = {
        "name": "Loudness",
        "values": N,
        "specific values": N_spec,
        "freqs": bark_axis,
    }

    # Check axis dimensions
    assert len(N) == len(time_axis)
    assert N_spec.shape[1] == len(time_axis)
    assert N_spec.shape[0] == len(bark_axis)

    # Check ISO 532-1 compliance
    assert _check_compliance(loudness, signal, "./tests/output/")


# test de la fonction
if __name__ == "__main__":
    test_loudness_zwtv()



import numpy as np


def _nonlinearity(p):
    """Apply the compressive nonlinearity of the auditory system to the
    rectified band pass signal rms values according to ECMA 418-2 section 5.1.7

    Parameters
    ----------
    p: numpy.array
        RMS value of the rectified band pass signal blocks

    Returns
    -------
    a_prime: numpy.array
        Specific Loudness [sone_HMS per Bark].

    """
    p_0 = 2e-5
    # c_N: In sones/bark
    c_N = 0.0217406
    alpha = 1.50
    v_i = np.array(
        [1.0, 0.6602, 0.0864, 0.6384, 0.0328, 0.4068, 0.2082, 0.3994, 0.6434]
    )
    thresh = np.array([15.0, 25.0, 35.0, 45.0, 55.0, 65.0, 75.0, 85.0])
    p_ti = p_0 * 10 ** (thresh / 20)

    a_prime = np.ones(p.shape)
    for i in range(8):
        a_prime *= (1 + (p / p_ti[i]) ** alpha) ** ((v_i[i + 1] - v_i[i]) / alpha)
    a_prime *= c_N * p / p_0

    return a_prime


# -*- coding: utf-8 -*-
"""
@date Created on Fri May 22 2020
@author martin_g for Eomys
"""

# Standard library imports
import math

# Third party import
import numpy as np


def _nl_loudness(core_loudness):
    """Simulate the nonlinear temporal decay of the hearing system

    Parameters
    ----------
    core_loudness : numpy.ndarray
        Core loudness

    Outputs
    -------
    nl_loudness :  numpy.ndarray
        Loudness with non linear temporal decay
    """
    # Initialization
    sample_rate = 2000
    nl_loudness = np.copy(core_loudness)
    # Factor for virtual upsampling/inner iterations
    nl_iter = 24
    # Time constants for non_linear temporal decay
    t_short = 0.005
    t_long = 0.015
    t_var = 0.075
    # Initializes constants B and states of capacitors C1 and C2
    delta_t = 1 / (sample_rate * nl_iter)
    P = (t_var + t_long) / (t_var * t_short)
    Q = 1 / (t_short * t_var)
    lambda_1 = -P / 2 + math.sqrt(P * P / 4 - Q)
    lambda_2 = -P / 2 - math.sqrt(P * P / 4 - Q)
    den = t_var * (lambda_1 - lambda_2)
    e1 = math.exp(lambda_1 * delta_t)
    e2 = math.exp(lambda_2 * delta_t)
    B = [
        (e1 - e2) / den,
        ((t_var * lambda_2 + 1) * e1 - (t_var * lambda_1 + 1) * e2) / den,
        ((t_var * lambda_1 + 1) * e1 - (t_var * lambda_2 + 1) * e2) / den,
        (t_var * lambda_1 + 1) * (t_var * lambda_2 + 1) * (e1 - e2) / den,
        math.exp(-delta_t / t_long),
        math.exp(-delta_t / t_var),
    ]
    nl_lp = {"B": B}
    #nl_lp["uo_last"] = 0
    #nl_lp["u2_last"] = 0

    delta = np.copy(core_loudness,)
    delta = np.roll(delta, -1, axis=1)
    delta[:, -1] = 0
    delta = (delta - nl_loudness)/nl_iter
    ui_delta = np.zeros(core_loudness.size*nl_iter).reshape(
        core_loudness.shape[0], core_loudness.shape[1], nl_iter)
    ui_delta[:, :, 0] = core_loudness

    for i_in in np.arange(1, nl_iter):
        ui_delta[:, :, i_in] = ui_delta[:, :, i_in-1] + delta

    ui_delta = ui_delta.reshape(
        core_loudness.shape[0], core_loudness.shape[1]*nl_iter)
    uo_mat = np.copy(ui_delta,)
    # create u2_mat (equivalent to u2 last) and fill the first col.
    u2_mat = np.zeros(uo_mat.shape[0]*uo_mat.shape[1]
                      ).reshape(uo_mat.shape[0], uo_mat.shape[1])
    mask = core_loudness[:, 0] >= 1e-5
    u2_mat[mask, 0] = core_loudness[mask, 0]*(1 - nl_lp["B"][5])

    '''
    Truth Table for u2 & uo
    u2 = nl_lp["uo_last"] * nl_lp["B"][0] - nl_lp["u2_last"] * nl_lp["B"][1]
    u2' = (nl_lp["u2_last"] - ui) * nl_lp["B"][5] + ui
    ui < uo(j-1) | uo(j-1) > u2(j-1) | uo < ui  |u2 > uo  |uo' < ui  |abs(ui -  uo(j-1)) < 1e-5 | ui > u2(j-1) | uo                                      |  u2
    ---------------------------------------------------------------
        Truth           Truth           Truth      Truth       --             --                                 ui                                       ui
        Truth           Truth           False      Truth       --             --                                 uo=f(uo(j-1),u2(j-1),B(2),B(3))          uo=f(uo(j-1),u2(j-1),B(2),B(3))
        Truth           Truth           Truth      False       --             --                                 ui                                       u2=f(uo(j-1),u2(j-1),B(0),B(1))
        Truth           Truth           False      False       --             --                                 uo=f(uo(j-1),u2(j-1),B(2),B(3))          u2=f(uo(j-1),u2(j-1),B(0),B(1))
        Truth           False                                  Truth                                             ui                                       ui
        Truth           False                                  False                                             uo=f(uo(j-1),B(4)                        uo=f(uo(j-1),B(4) 
        False                                                                Truth                  Truth        ui                                       u2=f(u2(j-1),B(5),ui(j)
        False                                                                Truth                  False        ui                                       ui
        False                                                                False                  Truth        ui                                       u2=f(u2(j-1),B(5),ui(j)
        False                                                                False                  False        ui                                       u2=f(u2(j-1),B(5),ui(j)
 
    '''
    for col in np.arange(core_loudness.shape[1]*nl_iter):
        uo2 = uo_mat[:, col-1] * nl_lp["B"][2] - \
            u2_mat[:, col-1] * nl_lp["B"][3]
        mask = np.logical_and(
            uo_mat[:, col-1] > u2_mat[:, col-1], uo2 >= ui_delta[:, col])
        uo_mat[mask, col] = uo2[mask]  # in case uo higher than ui

        uo2 = uo_mat[:, col-1] * nl_lp["B"][4]
        mask = np.logical_and(uo_mat[:, col-1] <=
                              u2_mat[:, col-1], uo2 >= ui_delta[:, col])
        uo_mat[mask, col] = uo2[mask]  # in case uo_2 higher than ui

        u2_mat[:, col] = uo_mat[:, col]  # higher than uo
        u22 = uo_mat[:, col-1] * nl_lp["B"][0] - \
            u2_mat[:, col-1] * nl_lp["B"][1]
        mask = np.logical_and(np.logical_and(
            ui_delta[:, col] < uo_mat[:, col-1], uo_mat[:, col-1] > u2_mat[:, col-1]), u22 <= uo_mat[:, col])
        u2_mat[mask, col] = u22[mask]  # in case u22 lower than uo_last

        u2_2 = (u2_mat[:, col-1] - ui_delta[:, col]) * \
            nl_lp["B"][5] + ui_delta[:, col]
        mask = np.logical_and(ui_delta[:, col] >= uo_mat[:, col-1], np.logical_not(np.logical_and(
            abs(ui_delta[:, col] - uo_mat[:, col-1]) < 1e-5,  uo_mat[:, col] <= u2_mat[:, col-1])))
        u2_mat[mask, col] = u2_2[mask]  # lower than ui

    nl_loudness = uo_mat.reshape(
        core_loudness.shape[0], core_loudness.shape[1], nl_iter)[:, :, 0]
    uo_mat = uo_mat.reshape(
        core_loudness.shape[0], core_loudness.shape[1]*nl_iter)

    return nl_loudness



# -*- coding: utf-8 -*-
"""
Created on Fri Dec 18 15:03:31 2020

@author: wantysal
"""

import numpy as np
from numpy.fft import fft

# Local functions imports
from mosqito.utils.conversion import amp2db
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._critical_band import _critical_band
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._screening_for_tones import (
    _screening_for_tones,
)
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._find_highest_tone import (
    _find_highest_tone,
)
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._peak_level import (
    _peak_level,
)


def _tnr_main_calc(signal, fs):
    """
        Calculation of the tone-to noise ratio according to the method described
        in ECMA 74, annex D.
        The method to find the tonal candidates is the one described by Wade Bray
        in 'Methods for automating prominent tone evaluation and for considerin
        variations with time or other reference quantities'
        The total value calculated, T-TNR, is calculated according to ECMA TR/108

    Parameters
    ----------
    signal : numpy.array
        time history values
    fs : integer
        sampling frequency

    Output
    ------
    tones_freqs : list of float
        frequency of the tones
    tnr : list of float
        TNR value calculated for each tone
    prominence : list of boolean
        prominence criteria as described in ECMA 74
    t_tnr : list of float
        sum of the specific TNR
    """

    #### Spectrum creation #######################################################

    n = len(signal)
    window = np.hanning(n)
    window = window / np.sum(window)

    # Creation of the spectrum by FFT
    spectrum = fft(signal * window) * 1.42

    # Conversion into dB level
    module = np.abs(spectrum)
    spectrum_db = amp2db(module, ref=0.00002)

    # Frequency axis of interest
    freq_axis = np.arange(0, int(n / 2), 1) * (fs / n)
    freq_index = np.where((freq_axis > 89.1) & (freq_axis < 11200))[0]
    freqs = freq_axis[freq_index]
    spec_db = spectrum_db[freq_index]

    #### Screening to find the potential tonal components ########################

    peak_index = _screening_for_tones(freqs, spec_db, "smoothed", 90, 11200)
    nb_tones = len(peak_index)

    #### Evaluation of each candidate ############################################

    # Initialization of the results lists
    tnr = np.array(())
    tones_freqs = np.array(())
    prominence = []

    # Each candidate is studied and then deleted from the list until all have been treated
    while nb_tones > 0:
        ind = peak_index[0]
        if len(peak_index) > 1:
            ind_p, ind_s, peak_index, nb_tones = _find_highest_tone(
                freqs, spec_db, peak_index, nb_tones, ind
            )
        else:
            ind_p = ind
            ind_s = None

        # multiple tones in a critical band
        if ind_s != None:
            fp = freqs[ind_p]
            fs = freqs[ind_s]

            # proximity criterion
            delta_f = 21 * 10 ** ((1.2 * (np.abs(np.log10(fp / 212))) ** 1.8))
            if np.abs(fs - fp) < delta_f:

                # tone SPL
                Lp = _peak_level(freqs, spec_db, ind_p)
                Ls = _peak_level(freqs, spec_db, ind_s)

                Lt = 10 * np.log10(((10 ** (Lp / 10) + 10 ** (Ls / 10))))

                # total SPL in the critical band
                f1, f2 = _critical_band(fp)
                low_limit_idx = np.argmin(np.abs(freqs - f1))
                high_limit_idx = np.argmin(np.abs(freqs - f2))

                spec_sum = sum(10 ** (spec_db[low_limit_idx:high_limit_idx] / 10))
                Ltot = 10 * np.log10(spec_sum)

                # suppression of the second highest tone from the list of tones
                sup = np.where(peak_index == ind_s)[0]
                peak_index = np.delete(peak_index, sup)
                nb_tones -= 1

                delta_ft = 2 * (freq_axis[1] - freq_axis[0])

            else:
                # the two highest tones are not close enough to be considered as one
                # tone SPL
                Lt = spec_db[ind_p]

                # total SPL in the critical band
                f1, f2 = _critical_band(freqs[ind_p])
                low_limit_idx = np.argmin(np.abs(freqs - f1))
                high_limit_idx = np.argmin(np.abs(freqs - f2))

                spec_sum = sum(10 ** (spec_db[low_limit_idx:high_limit_idx] / 10))
                Ltot = 10 * np.log10(spec_sum)

                delta_ft = freqs[1] - freqs[0]

        # single tone in a critical band
        else:
            # tone SPL
            Lt = _peak_level(freqs, spec_db, ind_p)

            # total SPL in the critical band
            f1, f2 = _critical_band(freqs[ind_p])
            low_limit_idx = np.argmin(np.abs(freqs - f1))
            high_limit_idx = np.argmin(np.abs(freqs - f2))

            spec_sum = sum(10 ** (spec_db[low_limit_idx:high_limit_idx] / 10))
            Ltot = 10 * np.log10(spec_sum)

            delta_ft = freqs[1] - freqs[0]

        # SPL of the masking noise
        delta_fc = f2 - f1
        delta_ftot = freq_axis[high_limit_idx] - freq_axis[low_limit_idx]
        Ln = 10 * np.log10(10 ** (Ltot / 10) - 10 ** (Lt / 10)) + 10 * np.log10(
            delta_fc / (delta_ftot - delta_ft)
        )

        # Tone-to-noise ratio
        f = freqs[ind_p]
        delta_t = Lt - Ln
        if delta_t > 0:
            tones_freqs = np.append(tones_freqs, f)
            tnr = np.append(tnr, delta_t)

            # Prominence criteria
            if f >= 89.1 and f < 1000:
                if delta_t >= 8 + 8.33 * np.log10(1000 / f):
                    prominence.append(True)
                else:
                    prominence.append(False)
            elif f >= 1000 and f <= 11200:
                if delta_t >= 8:
                    prominence.append(True)
                else:
                    prominence.append(False)

        # suppression from the list of tones
        sup = np.where(peak_index == ind_p)[0]
        peak_index = np.delete(peak_index, sup)
        nb_tones -= 1

    if sum(np.power(10, (tnr / 10))) != 0:
        t_tnr = 10 * np.log10(sum(np.power(10, (tnr / 10))))
    else:
        t_tnr = 0

    return tones_freqs, tnr, prominence, t_tnr



# -*- coding: utf-8 -*-
"""
@date Created on Fri Feb 28 2020
@author martin_g for Eomys
"""

# Standard library imports
import math


def sone_to_phon(sone):
    """Calculate Loudness level [phons] from Loudness [sones]

    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).

    Parameters
    ----------
    N : float
        Loudness [sones]

    Outputs
    -------
    LN : float
        Loudness level [phons]
    """

    if sone < 1:
        phon = 40 * sone ** 0.35
        if phon < 3:
            phon = 3
    else:
        phon = 10 * math.log2(sone) + 40
    return phon



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 11:02:48 2020

@author: wantysal
"""

# Standard import
import numpy as np


# ----------------------------------Pa <-> dB-----------------------------------
def db2amp(dB, ref=1):
    """Linearisation of a SPL level in dB

    Parameters
    ----------
    dB : numpy.array
        dB values to be converted
    ref: float
        reference value
    """
    if ref == 0:
        raise ValueError("Reference must be different from 0")

    return np.power(10, 0.05 * dB) * ref


def amp2db(amp, ref=1):
    """Conversion of an amplitude value into dB

    Parameters:
    -----------
    amp: np.array
        amplitude values to be converted
    ref: float
        reference value

    """
    if ref == 0:
        raise ValueError("Reference must be different from 0")
    elif ref != 0:
        db = 20 * np.log10(amp / ref)

    return db


# ------------------------------Hertz <-> Bark----------------------------------


def bark2freq(bark_axis):
    """Frequency conversion from Bark to Hertz

     See E. Zwicker, H. Fastl: Psychoacoustics. Springer,Berlin, Heidelberg, 1990.
     The coefficients are linearly interpolated from the values given in table 6.1.

     Parameter
     ---------
     bark_axis : numpy.array
                 Bark frequencies to be converted

     Output
     ------
    freq_axis : numpy.array
                frequencies converted in Hertz

    """

    xp = np.arange(0, 25, 0.5)

    yp = np.array(
        [
            0,
            50,
            100,
            150,
            200,
            250,
            300,
            350,
            400,
            450,
            510,
            570,
            630,
            700,
            770,
            840,
            920,
            1000,
            1080,
            1170,
            1270,
            1370,
            1480,
            1600,
            1720,
            1850,
            2000,
            2150,
            2320,
            2500,
            2700,
            2900,
            3150,
            3400,
            3700,
            4000,
            4400,
            4800,
            5300,
            5800,
            6400,
            7000,
            7700,
            8500,
            9500,
            10500,
            12000,
            13500,
            15500,
            20000,
        ]
    )

    return np.interp(bark_axis, xp, yp)


def freq2bark(freq_axis):
    """Frequency conversion from Hertz to Bark

     See E. Zwicker, H. Fastl: Psychoacoustics. Springer,Berlin, Heidelberg, 1990.
     The coefficients are linearly interpolated from the values given in table 6.1.

     Parameter
     ---------
     freq_axis : numpy.array
                 Hertz frequencies to be converted

     Output
     ------
    bark_axis : numpy.array
               frequencies converted in Bark

    """

    xp = np.array(
        [
            0,
            50,
            100,
            150,
            200,
            250,
            300,
            350,
            400,
            450,
            510,
            570,
            630,
            700,
            770,
            840,
            920,
            1000,
            1080,
            1170,
            1270,
            1370,
            1480,
            1600,
            1720,
            1850,
            2000,
            2150,
            2320,
            2500,
            2700,
            2900,
            3150,
            3400,
            3700,
            4000,
            4400,
            4800,
            5300,
            5800,
            6400,
            7000,
            7700,
            8500,
            9500,
            10500,
            12000,
            13500,
            15500,
            20000,
        ]
    )

    yp = np.arange(0, 25, 0.5)

    return np.interp(freq_axis, xp, yp)


# -----------------------------------dB <-> dBA---------------------------------


def spectrum2dBA(spectrum, fs):
    """A_weighting dB ponderation of a spectrum according to CEI 61672:2014

    Third-octave spectrum are directly calculated, other are calculated
    using linear interpolation.

    Parameters
    ----------
    spectrum: numpy.array
              input spectrum
    fs: integer
        sampling frequency

    """

    # Ponderation coefficients from the standard
    A_standard = np.array(
        [
            -70.4,
            -63.4,
            -56.7,
            -50.5,
            -44.7,
            -39.4,
            -34.6,
            -30.2,
            -26.2,
            -22.5,
            -19.1,
            -16.1,
            -13.4,
            -10.9,
            -8.6,
            -6.6,
            -4.8,
            -3.2,
            -1.9,
            -0.8,
            0,
            0.6,
            1,
            1.2,
            1.3,
            1.2,
            1,
            0.5,
            -0.1,
            -1.1,
            -2.5,
            -4.3,
            -6.6,
            -9.3,
        ]
    )

    freq_standard = np.array(
        [
            10,
            12.5,
            16,
            20,
            25,
            31.5,
            40,
            50,
            63,
            80,
            100,
            125,
            160,
            200,
            250,
            315,
            400,
            500,
            630,
            800,
            1000,
            1250,
            1600,
            2000,
            2500,
            3150,
            4000,
            5000,
            6300,
            8000,
            10000,
            12500,
            16000,
            20000,
        ]
    )

    # Linear interpolation on the spectrum axis
    spectrum_freq_axis = np.linspace(0, int(fs / 2), spectrum.size)
    A_pond = np.interp(spectrum_freq_axis, freq_standard, A_standard)

    # Ponderation of the given spectrum
    spectrum_dBA = np.zeros(spectrum.shape)
    for i in range(spectrum.shape[0]):
        spectrum_dBA[i] = spectrum[i] + A_pond[i]

    return spectrum_dBA



# -*- coding: utf-8 -*-

import numpy as np

from mosqito.sq_metrics.sharpness.sharpness_din._weighting_fastl import x, y


def sharpness_din_from_loudness(N, N_specific, weighting="din", skip=0):
    """Acoustic sharpness calculation according to different methods:
        Aures, Von Bismarck, DIN 45692, Fastl

    Parameters:
    ----------
    N : float or numpy.ndarray
        The overall loudness [sones]. If array, size (Ntime,)
    N_specific : numpy.ndarray
        The specific loudness array [sones/bark], size (Nbark,) or (Nbark, Ntime)
    weighting : string
        To specify the weighting function used for the
        sharpness computation.'din' by default,'aures', 'bismarck','fastl'
    skip : float
        number of second to be cut at the beginning of the analysis

    Outputs
    ------
    S : float or numpy.ndarray
        sharpness value

    """

    # 1D-array => 2D-array
    if not isinstance(N, np.ndarray):
        N = np.array([N])
    elif N.ndim <= 1:
        N = N[np.newaxis, :]
    if N_specific.ndim <= 1:
        N_specific = N_specific[:, np.newaxis]

    # Bark axis
    z = np.linspace(0.1, 24, int(24 / 0.1))[:, np.newaxis]

    # weighting function
    if weighting == "din":
        g = np.ones(z.shape)
        g[z > 15.8] = 0.15 * np.exp(0.42 * (z[z > 15.8] - 15.8)) + 0.85
    elif weighting == "aures":
        g = 0.078 * (np.exp(0.171 * z) / z) * (N / np.log(N * 0.05 + 1))
    elif weighting == "bismarck":
        g = np.ones(z.shape)
        g[z > 15] = 0.2 * np.exp(0.308 * (z[z > 15] - 15)) + 0.8
    elif weighting == "fastl":
        g = np.interp(z, x, y)
    else:
        raise ValueError(
            "ERROR: weighting must be 'din', 'aures', 'bismarck' or 'fastl'"
        )

    S = np.zeros(N.shape)
    ind = np.where(N >= 0.1)[0]
    S[ind] = (
        0.11
        * np.sum(
            N_specific[:, ind] * g * z * 0.1,
            axis=0,
        )
        / N[ind]
    )

    if S.size == 1:
        S = float(S)
    else:
        S = np.squeeze(S)
    return S


# -*- coding: utf-8 -*-
"""
Created on Thu Dec 10 16:51:19 2020

@author: wantysal
"""


# Standard library imports
import numpy as np
import math

# Local functions imports
from mosqito.sq_metrics.tonality.tone_to_noise_ecma._tnr_main_calc import _tnr_main_calc


def tone_to_noise_ecma(is_stationary, signal, fs, prominence=True):
    """Computation of tone-to-noise ration according to ECMA-74, annex D.9
        The T-TNR value is calculated according to ECMA-TR/108

    Parameters
    ----------
    is_stationary : boolean
        True if the signal is stationary
    signal :numpy.array
        time signal values
    fs : integer
        sampling frequency
    prominence : boolean
        if True, the algorithm only returns the prominent tones, if False it returns all tones detected

    Output
    ------
    output = dict
    {    "name" : "tone-to-noise ratio",
         "time" : np.linspace(0, len(signal)/fs, num=nb_frame),
         "freqs" : <frequency of the tones>
         "values" : <TNR calculated value for each tone>
         "prominence" : <True or False according to ECMA criteria>
         "global value" : <sum of the specific TNR values>
            }


    """
    # Prominence criteria
    freqs = np.arange(90, 11200, 100)
    limit = np.zeros((len(freqs)))
    for i in range(len(freqs)):
        if freqs[i] >= 89.1 and freqs[i] < 1000:
            limit[i] = 8 + 8.33 * np.log10(1000 / freqs[i])
        if freqs[i] >= 1000 and freqs[i] < 11200:
            limit[i] = 8

    if is_stationary == True:
        tones_freqs, tnr, prom, t_tnr = _tnr_main_calc(signal, fs)

        tones_freqs = tones_freqs.astype(int)

        if prominence == True:
            output = {
                "name": "tone-to-noise ratio",
                "freqs": tones_freqs[prom],
                "values": tnr[prom],
                "prominence": True,
                "global value": t_tnr,
            }

        else:
            output = {
                "name": "tone-to-noise ratio",
                "freqs": tones_freqs,
                "values": tnr,
                "prominence": prom,
                "global value": t_tnr,
            }

    if is_stationary == False:
        # Signal cut in frames of 200 ms along the time axis
        n = 0.5 * fs
        nb_frame = math.floor(signal.size / n)
        time = np.linspace(0, len(signal) / fs, num=nb_frame)
        time = np.around(time, 1)

        # Initialization of the result arrays
        tones_freqs = np.zeros((nb_frame), dtype=list)
        tnr = np.zeros((nb_frame), dtype=list)
        prom = np.zeros((nb_frame), dtype=list)
        t_tnr = np.zeros((nb_frame))

        # Calculate TNR values along time
        for i_frame in range(nb_frame):
            segment = signal[int(i_frame * n) : int(i_frame * n + n)]
            (
                tones_freqs[i_frame],
                tnr[i_frame],
                prom[i_frame],
                t_tnr[i_frame],
            ) = _tnr_main_calc(segment, fs)

        # Store the results in a time vs frequency array
        freqs = np.logspace(np.log10(90), np.log10(11200), num=1000)
        results = np.zeros((len(freqs), nb_frame))
        promi = np.zeros((len(freqs), nb_frame), dtype=bool)

        if prominence == True:

            for t in range(nb_frame):
                for f in range(len(tones_freqs[t])):
                    if prom[t][f] == True:
                        ind = np.argmin(np.abs(freqs - tones_freqs[t][f]))
                        results[ind, t] = tnr[t][f]
                        promi[ind, t] = True
        else:
            for t in range(nb_frame):
                for f in range(len(tones_freqs[t])):
                    ind = np.argmin(np.abs(freqs - tones_freqs[t][f]))
                    results[ind, t] = tnr[t][f]
                    promi[ind, t] = prom[t][f]

        output = {
            "name": "tone-to-noise ratio",
            "time": time,
            "freqs": freqs,
            "values": results,
            "prominence": promi,
            "global value": t_tnr,
        }

    return output



# -*- coding: utf-8 -*-

# Third party imports
from time import time
import numpy as np

# Local application imports
from mosqito.utils import time_segmentation
from mosqito.sq_metrics.loudness.loudness_zwst.loudness_zwst import loudness_zwst


def loudness_zwst_perseg(signal, fs, nperseg=4096, noverlap=None, field_type="free"):
    """Zwicker-loudness calculation for stationary signals

    Calculates the acoustic loudness according to Zwicker method for
    stationary signals per signal segment.
    Normatice reference:
        ISO 532:1975 (method B)
        DIN 45631:1991
        ISO 532-1:2017 (method 1)
    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).
    Note that due to normative continuity, as defined in the
    preceeding standards, the method is in accordance with
    ISO 226:1987 equal loudness contours (instead of ISO 226:2003)

    Parameters
    ----------
    signal : numpy.array
        Time signal values [Pa].
    fs : integer
        Sampling frequency.
    nperseg: int, optional
        Length of each segment. Defaults to 4096.
    noverlap: int, optional
        Number of points to overlap between segments.
        If None, noverlap = nperseg / 2. Defaults to None.
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse").

    Outputs
    -------
    N : float
        The overall loudness array [sones], size (Ntime,)
    N_specific : numpy.ndarray
        The specific loudness array [sones/bark], size (Nbark, Ntime)
    bark_axis: numpy.array
        The Bark axis array, size (Nbark,)
    time_axis: numpy.array
        The time axis array, size (Ntime,) or None

    """

    # Time signal segmentation
    signal, time_axis = time_segmentation(signal, fs, nperseg, noverlap)

    # Compute loudness
    N, N_specific, bark_axis = loudness_zwst(signal, fs, field_type="free")

    return N, N_specific, bark_axis, time_axis



# -*- coding: utf-8 -*-

import numpy as np
import scipy.signal as sp_signal

from mosqito.sq_metrics.loudness.loudness_ecma._ear_filter_design import (
    _ear_filter_design,
)
from mosqito.sq_metrics.loudness.loudness_ecma._auditory_filters_centre_freq import (
    _auditory_filters_centre_freq,
)
from mosqito.sq_metrics.loudness.loudness_ecma._gammatone import _gammatone
from mosqito.utils import time_segmentation


def _rectified_band_pass_signals(sig, sb=2048, sh=1024):
    """Compute the rectified band-pass signals as per Clause 5.1.2 to 5.1.5 of
    ECMA-418-2:2020

    Calculation of the rectified band-pass signals along the 53 critical band rates
    scale. Each band pass signal is segmented into time blocks according to sb and sh

    Parameters
    ----------
    signal: numpy.array
        'Pa', time signal values. The sampling frequency of the signal must be 48000 Hz.
    sb: int or list of int
        block size.
    sh: int or list of int
        Hop size.
    Returns
    -------
    block_array_rect: list of numpy.array
        rectified band-pass signals
    """

    if isinstance(sb, int):
        sb = sb * np.ones(53, dtype=int)
    elif len(sb) != 53:
        raise ValueError("ERROR: len(sb) shall be either 1 or 53")
    if isinstance(sh, int):
        sh = sh * np.ones(53, dtype=int)
    elif len(sh) != 53:
        raise ValueError("ERROR: len(sh) shall be either 1 or 53")

    # OUTER AND MIDDLE EAR FILTERING (5.1.2)

    sos_ear = _ear_filter_design()
    signal_filtered = sp_signal.sosfilt(sos_ear, sig, axis=0)

    # AUDITORY FILTERING BANK (5.1.3)

    # Order of the Outer and Middle ear filter
    filter_order_k = 5
    # Sampling frequency
    fs = 48000.00
    # Auditory filters centre frequencies
    centre_freq = _auditory_filters_centre_freq()

    block_array_rect = []
    for band_number in range(53):
        bm_mod, am_mod = _gammatone(centre_freq[band_number], k=filter_order_k, fs=fs)

        band_pass_signal = (
            2.0
            * (
                sp_signal.lfilter(
                    bm_mod,
                    am_mod,
                    signal_filtered,
                    axis=0,
                )
            ).real
        )

        # SEGMENTATION OF THE SIGNAL INTO BLOCKS (5.1.4)

        # The segmentation of the signal is done in order to obtain results for intervals of time, not for the whole
        # duration of the signal. The reason behind this decision resides in the fact that processing the signal in its
        # full length at one time could end up in imprecise results. By using a "for loop", we are able to decompose the
        # signal array "band_pass_signal_hr" into blocks. "sb_array" is the block size which changes depending on the
        # "band_number" in which we are processing the signal. "sh_array" is the step size, the time shift to the next
        # block.

        block_array, _ = time_segmentation(
            band_pass_signal, fs, sb[band_number], sh[band_number], is_ecma=True
        )
        block_array = block_array.T

        # RECTIFICATION (5.1.5)

        # This part acts as the activation of the auditory nerves when the basilar membrane vibrates in a certain
        # direction. In order to rectify the signal we are using "np.clip" which establish a minimum and a maximum value
        # for the signal. "a_min" is set to 0 float, while "a_max" is set to "None" in order to consider the positive
        # value of the signal.

        block_array_rect.append(np.clip(block_array, a_min=0.00, a_max=None))

    return block_array_rect


from mosqito.sq_metrics.loudness.loudness_ecma.loudness_ecma import loudness_ecma
from mosqito.sq_metrics.loudness.loudness_zwst.loudness_zwst import loudness_zwst
from mosqito.sq_metrics.loudness.loudness_zwst.loudness_zwst_perseg import (
    loudness_zwst_perseg,
)
from mosqito.sq_metrics.loudness.loudness_zwtv.loudness_zwtv import loudness_zwtv
from mosqito.sq_metrics.loudness.utils.equal_loudness_contours import (
    equal_loudness_contours,
)
from mosqito.sq_metrics.tonality.prominence_ratio_ecma.prominence_ratio_ecma import (
    prominence_ratio_ecma,
)
from mosqito.sq_metrics.tonality.tone_to_noise_ecma.tone_to_noise_ecma import (
    tone_to_noise_ecma,
)
from mosqito.sq_metrics.roughness.roughness_dw.roughness_dw import (
    roughness_dw,
)
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din import sharpness_din
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_from_loudness import (
    sharpness_din_from_loudness,
)
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_perseg import (
    sharpness_din_perseg,
)
from mosqito.sq_metrics.loudness.utils.sone_to_phon import sone_to_phon



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 16 09:20:41 2020

@author: wantysal
"""

import numpy as np

# Optional package import
try:
    import pytest
except ImportError:
    raise RuntimeError("In order to perform the tests you need the 'pytest' package.")


# Local application imports
from mosqito.utils import load
from mosqito.sq_metrics import sharpness_din
from mosqito.sq_metrics import sharpness_din_perseg


@pytest.mark.sharpness_din  # to skip or run sharpness test
def test_sharpness_din():
    """Test function for the sharpness calculation of an audio signal

    Test function for the function "comp_roughness" with 'din' method.
    The input signals come from DIN 45692_2009E. The compliance is assessed
    according to chapter 6 of the standard.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """

    # Input signal from DIN 45692_2009E
    sig, fs = load("tests/input/broadband_570.wav", wav_calib=1)

    # Compute sharpness
    sharpness = sharpness_din(sig, fs, weighting="aures")
    sharpness = sharpness_din(sig, fs, weighting="bismarck")
    sharpness = sharpness_din(sig, fs, weighting="fastl")
    sharpness = sharpness_din(sig, fs, weighting="din")

    # Check that the value is within the desired values +/- 5%
    # as per DIN 45692_2009E (chapter 6)
    np.testing.assert_allclose(sharpness, 2.85, rtol=0.05)


@pytest.mark.sharpness_din  # to skip or run sharpness test
def test_sharpness_din_perseg():
    """Test function for the sharpness calculation of an audio signal

    Test function for the function "comp_roughness" with 'din' method.
    The input signals come from DIN 45692_2009E. The compliance is assessed
    according to chapter 6 of the standard.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """

    # Input signal from DIN 45692_2009E
    sig, fs = load("tests/input/broadband_570.wav", wav_calib=1)

    # Compute sharpness
    sharpness, _ = sharpness_din_perseg(sig, fs, nperseg=2 ** 14, weighting="aures")
    sharpness, _ = sharpness_din_perseg(sig, fs, nperseg=2 ** 14, weighting="bismarck")
    sharpness, _ = sharpness_din_perseg(sig, fs, nperseg=2 ** 14, weighting="fastl")
    sharpness, time_axis = sharpness_din_perseg(
        sig, fs, nperseg=2 ** 14, weighting="din"
    )

    # Check that the value is within the desired values +/- 5%
    # as per DIN 45692_2009E (chapter 6)
    np.testing.assert_allclose(sharpness, 2.85, rtol=0.05)


# test de la fonction
if __name__ == "__main__":
    test_sharpness_din()
    test_sharpness_din_perseg()



# -*- coding: utf-8 -*-
"""
Created on Mon Feb  8 10:41:09 2021

@author: wantysal
"""

# Optional package import
try:
    import pytest
except ImportError:
    raise RuntimeError(
        "In order to perform the tests you need the 'pytest' package."
        )


# Local application imports
from mosqito.utils import load
from mosqito.sq_metrics import prominence_ratio_ecma


@pytest.mark.pr  # to skip or run PR test
def test_pr_ecma():
    """Test function for the prominence ratio calculation of an audio signal

    Validation function for the Audio_signal class "tone_to_noise_ecma" method with signal array
    as input. The input signals are generated using audacity.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for prominence ratio calculation
    # signals generated using audacity : white noise + tones at 200 and 2000 Hz
    # the first one is stationary, the second is time-varying
    signal = []

    signal.append(
        {
            "is_stationary": True,
            "data_file": "tests/input/white_noise_442_1768_Hz_stationary.wav",
        }
    )

    signal.append(
        {
            "is_stationary": False,
            "data_file": "tests/input/white_noise_442_1768_Hz_varying.wav",
        }
    )

    for i in range(len(signal)):
        # Load signal
        audio, fs = load(signal[i]["data_file"])
        # Compute tone-to-noise ratio
        pr = prominence_ratio_ecma(
            signal[i]["is_stationary"], audio, fs, prominence=True
        )


if __name__ == "__main__":
    test_pr_ecma()



import numpy as np


def _getFrequencies(fstart, fend, b, G=10, fr=1000):
    """
    Calculate octave filter specifications
    :param fstart: start frequency
    :param fend: end frequency
    :param b: number of bands pr octave
    :param G: base
    :param fr: reference frequency
    :return: filter specification dict
    """

    # frequency matrix
    freqs = np.empty((0, 3))

    if G == 10:
        G = 10 ** (3 / 10)  # Base ten
    elif G == 2:
        G = 2
    else:
        print("The base system is not permitted. G must be 10 or 2")
        # raise

    x = -1000
    f2 = 0
    while f2 <= fend:
        # Exact midband frequencies
        if b % 2 == 0:  # even
            fm = (G ** ((2 * x - 59) / (2 * b))) * (fr)
        else:  # odd
            fm = (G ** ((x - 30) / b)) * (fr)
        # Bandedge frequencies
        f1 = (G ** (-1 / (2 * b))) * (fm)
        f2 = (G ** (1 / (2 * b))) * (fm)
        if f2 >= fstart:
            freqs = np.append(freqs, np.array([[f1, fm, f2]]), axis=0)
        x += 1
    return {"f": freqs, "b": b, "G": G}



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 08:59:34 2020

@author: wantysal
"""

# Standard library imports
import numpy as np
from scipy.io import wavfile, loadmat
from scipy.signal import resample

# Optional package import
try:
    import pyuff
except ImportError:
    pyuff = None


def load(file, wav_calib=None, mat_signal="", mat_fs=""):
    """Extract the signal and its time axis from .wav or .uff file,
    resample the signal to 48 kHz, and affects its sampling frequency
    and time signal values.

    Parameters
    ----------
    file : string
        string path to the signal file
    wav_calib : float, optional
        Wav file calibration factor [Pa/FS]. Level of the signal in Pa_peak
        corresponding to the full scale of the .wav file. If None, a
        calibration factor of 1 is considered. Default to None.
    mat_signal : string
        in case of a .mat file, name of the signal variable
    mat_fs : string
        in case of a .mat file, name of the sampling frequency variable

    Outputs
    -------
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency
    """

    # load the .wav file content
    if file[-3:] == "wav" or file[-3:] == "WAV":
        fs, signal = wavfile.read(file)

        # manage multichannel files
        if signal.ndim > 1:
            signal = signal[:, 0]
            print("[Info] Multichannel signal loaded. Keeping only first channel")

        # calibration factor for the signal to be in Pa
        if wav_calib is None:
            wav_calib = 1
            print("[Info] A calibration of 1 Pa/FS is considered")
        if isinstance(signal[0], np.int16):
            signal = wav_calib * signal / (2 ** 15 - 1)
        elif isinstance(signal[0], np.int32):
            signal = wav_calib * signal / (2 ** 31 - 1)
        elif isinstance(signal[0], np.float):
            signal = wav_calib * signal

    # load the .uff file content
    elif file[-3:].lower() == "uff" or file[-3:].lower() == "unv":
        data = _uff_load(file)

        # extract the signal values
        signal = data["data"]

        # calculate the sampling frequency
        fs = int(1 / data["abscissa_inc"])

    # load the .mat file content
    elif file[-3:] == "mat":
        matfile = loadmat(file)

        # extract the signal values and sampling frequency
        signal = matfile[mat_signal][:, 0]
        fs = matfile[mat_fs]
        fs = fs[:, 0]

    else:
        raise ValueError("""ERROR: only .wav .mat or .uff files are supported""")

    # resample to 48kHz to allow calculation
    if fs != 48000:
        signal = resample(signal, int(48000 * len(signal) / fs))
        fs = 48000
        print("[Info] Signal resampled to 48 kHz to allow calculation.")

    return signal, fs


def _uff_load(file):
    if pyuff is None:
        raise RuntimeError(
            "In order to load UFF files you need the 'pyuff' " "package."
        )
    uff_file = pyuff.UFF(file)
    data = uff_file.read_sets()
    return data



# -*- coding: utf-8 -*-
"""
@date Created on Mon Mar 23 2020
@author martin_g for Eomys
"""

# Optional package import
try:
    import pytest
except ImportError:
    raise RuntimeError("In order to perform the tests you need the 'pytest' package.")


import numpy as np

# Local application imports
from mosqito.utils import load
from mosqito.sq_metrics import loudness_zwst
from mosqito.sq_metrics import loudness_zwst_perseg
from mosqito.sq_metrics.loudness.loudness_zwst._main_loudness import (
    _main_loudness,
)
from mosqito.sq_metrics.loudness.loudness_zwst._calc_slopes import (
    _calc_slopes,
)
from validations.sq_metrics.loudness_zwst.validation_loudness_zwst import (
    _check_compliance,
)
from tests.input.Test_signal_1 import test_signal_1


@pytest.mark.loudness_zwst  # to skip or run only loudness zwicker stationary tests
def test_loudness_zwst_3oct():
    """Test function for the script loudness_zwst

    Test function for the script loudness_zwicker_stationary with
    third octave band spectrum as input. The input spectrum is
    provided by ISO 532-1 annex B2, the compliance is assessed
    according to section 5.1 of the standard. One .png compliance
    plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """

    # Target values
    target = {
        "data_file": "Test signal 1.txt",
        "N": 83.296,
        "N_specif_file": "tests/input/test_signal_1.csv",
    }

    #
    # Compute loudness
    Nm = _main_loudness(test_signal_1, field_type="free")
    N, N_specific = _calc_slopes(Nm)
    loudness = {"values": N, "specific values": N_specific}
    #
    # Asser complaiance
    tst = _check_compliance(loudness, target, "./tests/output/")
    assert tst


@pytest.mark.loudness_zwst  # to skip or run only loudness zwicker stationary tests
def test_loudness_zwst_wav():
    """Test function for the script loudness_zwicker_stationary

    Test function for the script loudness_zwicker_stationary with
    .wav file as input. The input file is provided by ISO 532-1 annex
    B3, the compliance is assessed according to section 5.1 of the
    standard. One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for stationary loudness
    # (from ISO 532-1 annex B3)
    signal = {
        "data_file": "tests/input/Test signal 5 (pinknoise 60 dB).wav",
        "N": 10.498,
        "N_specif_file": "tests/input/test_signal_5.csv",
    }

    # Load signal
    sig, fs = load(signal["data_file"], wav_calib=2 * 2 ** 0.5)

    # Compute Loudness
    N, N_specific, bark_axis = loudness_zwst(sig, fs)
    loudness = {
        "name": "Loudness",
        "values": N,
        "specific values": N_specific,
        "freqs": bark_axis,
    }

    # Check ISO 532-1 compliance
    assert _check_compliance(loudness, signal, "./tests/output/")


@pytest.mark.loudness_zwst  # to skip or run only loudness zwicker stationary tests
def test_loudness_zwst_44100Hz():
    """Test function for the script loudness_zwicker_stationary
    with input .wav file sampled at 44.1 kHz
    """
    # Test signal as input for stationary loudness
    # (from ISO 532-1 annex B3) resampled
    signal = {
        "data_file": "tests/input/Test signal 3 (1 kHz 60 dB)_44100Hz.wav",
        "N": 4.019,
        "N_specif_file": "tests/input/test_signal_3.csv",
    }

    # Load signal and compute third octave band spectrum
    sig, fs = load(signal["data_file"], wav_calib=2 * 2 ** 0.5)

    # Compute Loudness
    N, N_specific, bark_axis = loudness_zwst(sig, fs)
    loudness = {
        "name": "Loudness",
        "values": N,
        "specific values": N_specific,
        "freqs": bark_axis,
    }

    # Check ISO 532-1 compliance
    assert _check_compliance(loudness, signal, "./tests/output/")


@pytest.mark.loudness_zwst
def test_loudness_zwst_perseg():
    # Load signal
    sig, fs = load(
        "tests/input/Test signal 5 (pinknoise 60 dB).wav", wav_calib=2 * 2 ** 0.5
    )

    # Compute Loudness
    N, N_specific, bark_axis, time_axis = loudness_zwst_perseg(
        sig, fs, nperseg=8192 * 2, noverlap=4096
    )

    # Check that all values are within the desired values +/- 5%
    np.testing.assert_allclose(N, 10.498, rtol=0.05)


# test de la fonction
if __name__ == "__main__":
    # test_loudness_zwst_3oct()
    # test_loudness_zwst_wav()
    # test_loudness_zwst_44100Hz()
    test_loudness_zwst_perseg()


from mosqito.sound_level_meter.noct_spectrum.noct_spectrum import (
    noct_spectrum,
)
from mosqito.sq_metrics.loudness.loudness_ecma.loudness_ecma import loudness_ecma
from mosqito.sq_metrics.loudness.loudness_zwst.loudness_zwst import loudness_zwst
from mosqito.sq_metrics.loudness.loudness_zwst.loudness_zwst_perseg import (
    loudness_zwst_perseg,
)
from mosqito.sq_metrics.loudness.loudness_zwtv.loudness_zwtv import loudness_zwtv
from mosqito.sq_metrics.loudness.utils.equal_loudness_contours import (
    equal_loudness_contours,
)
from mosqito.sq_metrics.tonality.prominence_ratio_ecma.prominence_ratio_ecma import (
    prominence_ratio_ecma,
)
from mosqito.sq_metrics.tonality.tone_to_noise_ecma.tone_to_noise_ecma import (
    tone_to_noise_ecma,
)
from mosqito.sq_metrics.roughness.roughness_dw.roughness_dw import (
    roughness_dw,
)
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din import sharpness_din
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_from_loudness import (
    sharpness_din_from_loudness,
)
from mosqito.sq_metrics.sharpness.sharpness_din.sharpness_din_perseg import (
    sharpness_din_perseg,
)
from mosqito.sq_metrics.loudness.utils.sone_to_phon import sone_to_phon
from mosqito.utils.isoclose import isoclose
from mosqito.utils.load import load
from mosqito.utils.sine_wave_generator import sine_wave_generator
from mosqito.utils.time_segmentation import time_segmentation

# Colors and linestyles
COLORS = [
    "#69c3c5",
    "#9969c4",
    "#c46b69",
    "#95c469",
    "#2a6c6e",
]



from numpy import arange
from numpy import sinh


def _auditory_filters_centre_freq():
    """
    Auditory filter bank center frequencies generation

    This function generates the Auditory filter bank center frequencies according
    to ECMA-418-2 section 5.1.3.1 equation 6

    Parameters
    ----------

    Returns
    -------
    centre_freq: ndarray
        Vector of auditory filter bank center frequencies

    """

    band_number = arange(53)
    z_step_size = 0.50
    af_f0 = 81.9289  # ECMA-418-2
    c = 0.1618  # ECMA-418-2

    # Critical band rate scale
    z = (band_number + 1) * z_step_size
    var = c * z

    # Central frequency
    centre_freq = (af_f0 / c) * sinh(var)

    return centre_freq


if __name__ == "__main__":
    centre_freq = _auditory_filters_centre_freq()



import setuptools

# /!\ update before a release
MoSQITo_VERSION = "1.0.1"

# MoSQITo description
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

python_requires = ">= 3.5"

# MoSQITo dependencies
with open("requirements.txt", "r") as file:
    requirements = file.readlines()
    install_requires = "".join(
        requirements
    ).splitlines()  # remove endline in each element

tests_require = ["pytest>=5.4.1", "pandas", "openpyxl", "SciDataTool", "matplotlib"]
uff_require = ["pyuff"]
scidatatool_require = ["SciDataTool"]
all_require = tests_require + uff_require

setuptools.setup(
    name="mosqito",
    version=MoSQITo_VERSION,
    author="MoSQITo Developers",
    author_email="martin.glesser@eomys.com",
    description="Modular Sound Quality Integrated Toolbox",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Eomys/MoSQITo",
    download_url="https://github.com/Eomys/MoSQITo/archive/v{}.tar.gz".format(
        MoSQITo_VERSION
    ),
    packages=setuptools.find_packages(
        exclude=[
            "docs",
            "tutorials",
            "validations",
            "tests",
        ]
    ),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=python_requires,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={
        "testing": tests_require,
        "uff": uff_require,
        "SciDataTool": scidatatool_require,
        "all": all_require,
    },
)



# -*- coding: utf-8 -*-
"""
Created on Mon Dec 21 16:44:36 2020

@author: wantysal
"""
# Standard library import
import numpy as np

# Local import
from mosqito.sound_level_meter.noct_spectrum._getFrequencies import _getFrequencies


def _spectrum_smoothing(freqs_in, spec, noct, low_freq, high_freq, freqs_out):
    """
        Compute smoothed spectrum according to the n-th octave band chosen

    Parameters
    ----------
    freqs : numpy.array
        frequency axis
    spec : numpy.array
        spectrum in dB
    noct : integer
        n-th octave-band according to which smooth the spectrum
    low_freq : float
        lowest frequency of the n-th octave bands
    high_freq : float
        highest frequency of the n-th octave bands
    freqs_out : numpy.array
        frequency axis along which the smoothed spectrum is given

    Returns
    -------
    smoothed-spectrum : numpy.array
        smoothed spectrum along the given frequency axis

    """

    # n-th octave bands filter
    filter_freqs = _getFrequencies(
        low_freq, high_freq, noct, G=10, fr=1000)["f"]
    filter_freqs[len(filter_freqs) - 1, 2] = high_freq
    filter_freqs[0, 0] = low_freq

    # Smoothed spectrum creation
    nb_bands = filter_freqs.shape[0]
    smoothed_spectrum = np.zeros((nb_bands))
    i = 0
    # Each band is considered individually until all of them have been treated
    while nb_bands > 0:
        # Find the index of the spectral components within the frequency bin
        bin_index = np.where(
            (freqs_in >= filter_freqs[i, 0]) & (freqs_in <= filter_freqs[i, 2])
        )[0]
        # If the frequency bin is empty, it is deleted from the list
        if len(bin_index) == 0:
            smoothed_spectrum = np.delete(smoothed_spectrum, i, axis=0)
            filter_freqs = np.delete(filter_freqs, i, axis=0)
            nb_bands -= 1

        else:
            # The spectral components within the frequency bin are averaged on an energy basis
            spec_sum = 0
            for j in bin_index:
                spec_sum += 10 ** (spec[j] / 10)
            smoothed_spectrum[i] = 10 * np.log10(spec_sum / len(bin_index))
        nb_bands -= 1
        i += 1

    # Pose of the smoothed spectrum on the frequency-axis
    cor = []
    low = []
    high = []
    # Index of the lower, center and higher limit of each frequency bin into the original spectrum
    for i in range(len(filter_freqs)):
        cor.append(np.argmin(np.abs(freqs_out - filter_freqs[i, 1])))
        low.append(np.argmin(np.abs(freqs_out - filter_freqs[i, 0])))
        high.append(np.argmin(np.abs(freqs_out - filter_freqs[i, 2])))

    smooth_spec = np.zeros((spec.shape))
    for i in range(filter_freqs.shape[0]):
        smooth_spec[low[i]: high[i]] = smoothed_spectrum[i]

    return smooth_spec



# -*- coding: utf-8 -*-

# Third party imports
from time import time
import numpy as np

# Local application imports
from mosqito.sound_level_meter import noct_spectrum
from mosqito.sq_metrics.loudness.loudness_zwst._main_loudness import (
    _main_loudness,
)
from mosqito.sq_metrics.loudness.loudness_zwst._calc_slopes import (
    _calc_slopes,
)


def loudness_zwst(signal, fs, field_type="free"):
    """Zwicker-loudness calculation for stationary signals

    Calculates the acoustic loudness according to Zwicker method for
    stationary signals.
    Normatice reference:
        ISO 532:1975 (method B)
        DIN 45631:1991
        ISO 532-1:2017 (method 1)
    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).
    Note that due to normative continuity, as defined in the
    preceeding standards, the method is in accordance with
    ISO 226:1987 equal loudness contours (instead of ISO 226:2003)

    Parameters
    ----------
    signal : numpy.array
        Time signal values [Pa].
    fs : integer
        Sampling frequency.
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse").

    Outputs
    -------
    N : float
        The overall loudness array [sones], size (Ntime,)
    N_specific : numpy.ndarray
        The specific loudness array [sones/bark], size (Nbark, Ntime)
    bark_axis: numpy.array
        The Bark axis array, size (Nbark,)
    """

    #
    # Compute third octave band spectrum
    spec_third, _ = noct_spectrum(signal, fs, fmin=24, fmax=12600)
    spec_third = 20 * np.log10(spec_third / 2e-5)
    #
    # Compute main loudness
    Nm = _main_loudness(spec_third, field_type)
    #
    # Computation of specific loudness pattern and integration of overall
    # loudness by attaching slopes towards higher frequencies
    N, N_specific = _calc_slopes(Nm)
    #
    # Define Bark axis
    bark_axis = np.linspace(0.1, 24, int(24 / 0.1))

    return N, N_specific, bark_axis
